@@ -34,11 +34,10 @@ function (x, data=NULL, ..., subset, na.action = na.omit, scaled = TRUE){
 
 setMethod("ksvm",signature(x="vector"),
 function(x,...)
-  {
-    x <- t(t(x))
-    ret <- ksvm(x, ...)
-    return(ret)
-  })
+          { x <- t(t(x))
+            ret <- ksvm(x, ...)
+            return(ret)
+          })
     
 setMethod("ksvm",signature(x="matrix"),
 function (x,
@@ -79,18 +78,20 @@ function (x,
   n.action(ret) <- na.action
   
  if (is.null(type)) type(ret) <-
-    if (is.null(y)) "one-classification"
-    else if (is.factor(y)) "C-classification"
-    else "eps-regression"
+    if (is.null(y)) "one-svc"
+    else if (is.factor(y)) "C-svc"
+    else "eps-svr"
   
   if(!is.null(type))
-  type(ret) <- match.arg(type,c("C-classification",
-                          "nu-classification",
-                         "spoc-classification",
-                          "kbb-classification",
-                          "one-classification",
-                          "eps-regression",
-                          "nu-regression"))
+  type(ret) <- match.arg(type,c("C-svc",
+                                "nu-svc",
+                                "kbb-svc",
+                                "spoc-svc",
+                                "C-bsvc",
+                                "one-svc",
+                                "eps-svr",
+                                "eps-bsvr",
+                                "nu-svr"))
 
 
   ## scaling, subsetting, and NA handling
@@ -99,7 +100,6 @@ function (x,
     if(!is.null(y)) na.fail(y)
     x <- t(t(x)) ## make shure that col-indices are sorted
   }
-
   unscaledx <- x  
   x.scale <- y.scale <- NULL
  ## scaling
@@ -118,7 +118,7 @@ function (x,
       xtmp <- scale(x[,scaled])
       x[,scaled] <- xtmp
       x.scale <- attributes(xtmp)[c("scaled:center","scaled:scale")]
-      if (is.numeric(y)&&(type(ret)!="C-classification"&&type(ret)!="nu-classification"&&type(ret)!="spoc-classification")) {
+      if (is.numeric(y)&&(type(ret)!="C-svc"&&type(ret)!="nu-svc"&&type(ret)!="C-bsvc"&&type(ret)!="spoc-svc"&&type(ret)!="kbb-svc")) {
         y <- scale(y)
         y.scale <- attributes(y)[c("scaled:center","scaled:scale")]
         y <- as.vector(y)
@@ -132,13 +132,14 @@ function (x,
   if(is.character(kernel)){
     kernel <- match.arg(kernel,c("rbfdot","polydot","tanhdot","vanilladot","laplacedot","besseldot","anovadot"))
   
-  if((kernel == "tanhdot" || kernel == "vanilladot" || kernel == "polydot") && any(names(kpar) == "sigma") )
+  if((kernel == "tanhdot" || kernel == "vanilladot" || kernel == "polydot"|| kernel == "besseldot") &&  kpar=="automatic" )
     {
-      cat("no sigma parameter in this kernel. Did you set a hyperparameter for this kernel ?")
-      cat ("\n", " Setting default kernel parameters")
-      kpar <- NULL
+      cat (" Setting default kernel parameters ","\n")
+      kpar <- list()
     }
   }
+  
+  if (!is.function(kernel))
   if (!is.list(kpar)&&is.character(kpar)&&(class(kernel)=="rbfkernel" || class(kernel) =="laplacedot" || kernel == "laplacedot"|| kernel=="rbfdot")){
     kp <- match.arg(kpar,"automatic")
     if(kp=="automatic")
@@ -154,16 +155,8 @@ function (x,
 
   if(!is(kernel,"kernel")) stop("kernel must inherit from class `kernel'")
 
-
-
-  ##internal function used by smo_optim                                   
-  .kernelin <- function(v)
-    {
-      return(kernelMatrix(kernel,xd,xd[v,,drop=FALSE]))
-    }
-
-  if (!is.vector(y) && !is.factor (y) && !(type(ret)=="one-classification")) stop("y must be a vector or a factor.")
-  if ((type(ret) != "one-classification") && nrows != nrow(x)) stop("x and y don't match.")
+  if (!is.vector(y) && !is.factor (y) && !(type(ret)=="one-svc")) stop("y must be a vector or a factor.")
+  if ((type(ret) != "one-svc") && nrows != nrow(x)) stop("x and y don't match.")
   if(nu > 1|| nu <0) stop("nu must be between 0 an 1.")
   
   weightlabels <- NULL
@@ -171,7 +164,7 @@ function (x,
   weight <- 0
   wl <- 0
   ## in case of classification: transform factors into integers
-  if (type(ret) == "one-classification") # one class classification --> set dummy
+  if (type(ret) == "one-svc") # one class classification --> set dummy
     y <- 1
   else
     if (is.factor(y)) {
@@ -186,15 +179,16 @@ function (x,
       }
     }
     else {
-      if ((type(ret) =="C-classification" || type(ret) == "nu-classification" || type(ret) == "spoc-classification" || type(ret) == "kbb-classification") && any(as.integer (y) != y))
+      if ((type(ret) =="C-svc" || type(ret) == "nu-svc" ||type(ret) == "C-bsvc" || type(ret) == "spoc-svc" || type(ret) == "kbb-svc") && any(as.integer (y) != y))
         stop ("dependent variable has to be of factor or integer type for classification mode.")
 
-      if (type(ret) != "eps-regression" || type(ret) != "nu-regression")
+      if (type(ret) != "eps-svr" || type(ret) != "nu-svr"|| type(ret)!="eps-bsvr")
         lev(ret) <- sort(unique (y))
     }
  ## initialize    
   nclass(ret) <- length (unique(y))
   p <- 0
+  K <- 0 
   svindex <- problem <- NULL
   sigma <- 0.1
   degree <- offset <- scale <- 1
@@ -239,10 +233,14 @@ function (x,
            sigma <- kpar(kernel)$sigma
            degree <-  kpar(kernel)$degree
          },
-         ktype <- 4
+         {
+           ktype <- 4
+         }
          )
   prior(ret) <- list(NULL)
-  if(type(ret) == "C-classification"){
+
+
+  if(type(ret) == "C-svc"){
     indexes <- lapply(sort(unique(y)), function(kk) which(y == kk))
     for (i in 1:(nclass(ret)-1)) {
       jj <- i+1
@@ -251,7 +249,198 @@ function (x,
         ##prepare data
         li <- length(indexes[[i]])
         lj <- length(indexes[[j]])
-        xd <- matrix(0,(li+lj),dim(x)[2])
+        if(sparse)
+          xd <- as.matrix.csr(0,(li+lj),dim(x)[2])
+        else
+          xd <- matrix(0,(li+lj),dim(x)[2])
+        xdi <- 1:(li+lj) <= li
+        xd[xdi,rep(TRUE,dim(x)[2])] <- x[indexes[[i]],]
+        xd[xdi == FALSE,rep(TRUE,dim(x)[2])] <- x[indexes[[j]],]
+        if(y[indexes[[i]][1]] < y[indexes[[j]]][1])
+          {
+            yd <- c(rep(1,li),rep(-1,lj))
+            if(!is.null(class.weights)){
+            weight <- class.weights[weightlabels[c(i,j)]]
+            wl <- c(1,0)
+            nweights <- 2
+          }
+          }
+        else
+          {
+            yd <- c(rep(-1,li),rep(1,lj))
+            if(!is.null(class.weights)){
+            weight <- class.weights[weightlabels[c(j,i)]]
+            wl <- c(0,1)
+            nweigths <- 2
+          }
+          }
+       
+        boolabel <- yd >= 0
+        prior1 <- sum(boolabel)
+
+        md <- length(yd)
+        prior0 <- md - prior1
+        prior(ret)[[p]] <- list(prior1 = prior1, prior0 = prior0) 
+
+        if(ktype==4)
+          K <- kernelMatrix(kernel,xd)
+        
+        resv <- .Call("smo_optim",
+                      as.double(t(xd)),
+                      as.integer(nrow(xd)),
+                      as.integer(ncol(xd)),
+                      as.double(yd),
+                      as.double(K),
+                      
+                      as.integer(if (sparse) xd@ia else 0),
+                      as.integer(if (sparse) xd@ja else 0),
+                      as.integer(sparse),
+                      
+                      as.double(matrix(rep(-1,m))), ##linear term
+                      as.integer(ktype),
+                      as.integer(0), 
+                      as.double(C),
+                      as.double(nu),
+                      as.double(epsilon),
+                      as.double(sigma),
+                      as.double(degree),
+                      as.double(offset),
+                      as.integer(wl), ##weightlabel
+                      as.double(weight),
+                      as.integer(nweights),
+                      as.double(cache), 
+                      as.double(tol),
+                      as.integer(shrinking),
+                      PACKAGE="kernlab")
+       
+         alpha(ret)[p] <- list(resv[-(li+lj+1)])
+        ## nonzero alpha*y
+        coeff(ret)[p] <- list(alpha(ret)[[p]][alpha(ret)[[p]]>0]*yd[alpha(ret)[[p]]>0])
+        ## store SV indexes from current problem for later use in predict
+        alphaindex(ret)[p] <- list(c(indexes[[i]],indexes[[j]])[which(alpha(ret)[[p]]>0)])
+        ## save the indexes from all the SV in a vector (use unique?)
+        svindex <- c(svindex,alphaindex(ret)[[p]])
+        ## store betas in a vector 
+        b(ret) <- c(b(ret), resv[li+lj+1])
+        ## used to reconstruct indexes for the patterns matrix x from "indexes" (really usefull ?)
+        problem[p] <- list(c(i,j))
+        ##store C  in return object
+        param(ret)$C <- C
+##        margin(ret)[p] <- (min(kernelMult(kernel,xd[1:li,],,alpha(ret)[[p]][1:li])) - max(kernelMult(kernel,xd[li:(li+lj),],,alpha(ret)[[p]][li:(li+lj)])))/2
+      }
+    }
+  } 
+
+if(type(ret) == "nu-svc"){
+  indexes <- lapply(sort(unique(y)), function(kk) which(y == kk))
+    for (i in 1:(nclass(ret)-1)) {
+      jj <- i+1
+      for(j in jj:nclass(ret)) {
+        p <- p+1
+       ##prepare data
+        li <- length(indexes[[i]])
+        lj <- length(indexes[[j]])
+         if(sparse)
+          xd <- as.matrix.csr(0,(li+lj),dim(x)[2])
+        else
+          xd <- matrix(0,(li+lj),dim(x)[2])
+        xdi <- 1:(li+lj) <= li
+        xd[xdi,rep(TRUE,dim(x)[2])] <- x[indexes[[i]],]
+        xd[xdi == FALSE,rep(TRUE,dim(x)[2])] <- x[indexes[[j]],]
+        if(y[indexes[[i]][1]] < y[indexes[[j]]][1])
+          {
+            yd <- c(rep(1,li),rep(-1,lj))
+            if(!is.null(class.weights)){
+            weight <- class.weights[weightlabels[c(i,j)]]
+            wl <- c(1,0)
+            nweights <- 2
+          }
+          }
+        else
+          {
+            yd <- c(rep(-1,li),rep(1,lj))
+            if(!is.null(class.weights)){
+            weight <- class.weights[weightlabels[c(j,i)]]
+            wl <- c(0,1)
+            nweigths <- 2
+          }
+          }
+       
+        boolabel <- yd >= 0
+        prior1 <- sum(boolabel)
+        md <- length(yd)
+        prior0 <- md - prior1
+        prior(ret)[[p]] <- list(prior1 = prior1, prior0 = prior0)
+
+        if(ktype==4)
+             K <- kernelMatrix(kernel,xd)
+        
+        resv <- .Call("smo_optim",
+                      as.double(t(xd)),
+                      as.integer(nrow(xd)),
+                      as.integer(ncol(xd)),
+                      as.double(yd),
+                      as.double(K),
+                      as.integer(if (sparse) xd@ia else 0),
+                      as.integer(if (sparse) xd@ja else 0),
+                      as.integer(sparse),
+                      
+                      as.double(matrix(rep(-1,m))), #linear term
+                      as.integer(ktype),
+                      as.integer(1),
+                      as.double(C),
+                      as.double(nu),
+                      as.double(epsilon),
+                      as.double(sigma),
+                      as.double(degree),
+                      as.double(offset),
+                      as.integer(wl), #weightlabl.
+                      as.double(weight),
+                      as.integer(nweights),
+                      as.double(cache),
+                      as.double(tol), 
+                      as.integer(shrinking),
+                      PACKAGE="kernlab")
+        
+        alpha(ret)[p] <- list(resv[-(li+lj+1)])
+        ## alpha*y whithout including zeros (smaller kernel matrixes)
+        coeff(ret)[p] <- list(alpha(ret)[[p]][alpha(ret)[[p]]!= 0])
+        ##store SV indexes from current problem for later use in predict
+         alphaindex(ret)[p] <- list(c(indexes[[i]],indexes[[j]])[which(resv[-(li+lj+1)]!= 0)])
+        ##alphaindex(ret)[p] <- list(c(which(alpha(ret)[[p]][1:li]!=0)+li*(i-1),which(alpha(ret)[[p]][-(1:li)]!=0)+li+lj*(j-2)))
+        ##save the indexes from all the SV in a vector (use unique!)
+               svindex <- c(svindex,alphaindex(ret)[[p]])
+        ## store betas in a vector 
+               b(ret) <- c(b(ret), resv[li+lj+1])
+        ## used to reconstruct indexes for the patterns matrix x from "indexes"
+               problem[p] <- list(c(i,j))
+               param(ret)$nu <- nu
+        
+##        margin(ret)[p] <- (min(kernelMult(kernel,xd[1:li,],,alpha(ret)[[p]][1:li])) - max(kernelMult(kernel,xd[li:(li+lj),],,alpha(ret)[[p]][li:(li+lj)])))/2
+    
+      }
+    }
+  } 
+  
+
+
+  if(type(ret) == "C-bsvc"){
+     if(!is.null(class.weights))
+     weightedC <- class.weights[weightlabels] * rep(C,nclass(ret))
+    else
+      weightedC <- rep(C,nclass(ret)) 
+    indexes <- lapply(sort(unique(y)), function(kk) which(y == kk))
+    for (i in 1:(nclass(ret)-1)) {
+      jj <- i+1
+      for(j in jj:nclass(ret)) {
+        p <- p+1
+        ##prepare data
+        li <- length(indexes[[i]])
+        lj <- length(indexes[[j]])
+        if(sparse)
+          xd <- as.matrix.csr(0,(li+lj),dim(x)[2])
+        else
+          xd <- matrix(0,(li+lj),dim(x)[2])
         xdi <- 1:(li+lj) <= li
         xd[xdi,rep(TRUE,dim(x)[2])] <- x[indexes[[i]],]
         xd[xdi == FALSE,rep(TRUE,dim(x)[2])] <- x[indexes[[j]],]
@@ -280,43 +469,47 @@ function (x,
         prior0 <- md - prior1
         prior(ret)[[p]] <- list(prior1 = prior1, prior0 = prior0) 
 
-        resv <- .Call("smo_optim",
+           if(ktype==4)
+          K <- kernelMatrix(kernel,xd)
+        
+        resv <- .Call("tron_optim",
                       as.double(t(xd)),
                       as.integer(nrow(xd)),
                       as.integer(ncol(xd)),
                       as.double(yd),
-
-                      
+                      as.double(K),
                       as.integer(if (sparse) xd@ia else 0),
                       as.integer(if (sparse) xd@ja else 0),
                       as.integer(sparse),
-                      
-                      as.double(matrix(rep(-1,m))), ##linear term
+                      as.integer(2),
+                      as.double(0), ##countc
                       as.integer(ktype),
-                      as.integer(0), 
+                      as.integer(5), 
                       as.double(C),
-                      as.double(nu),
                       as.double(epsilon),
                       as.double(sigma),
                       as.double(degree),
                       as.double(offset),
+                      as.double(1),  ##  cost value of alpha seeding
+                      as.double(2),  ## step value of alpha seeding
                       as.integer(wl), ##weightlabel
                       as.double(weight),
                       as.integer(nweights),
+                      as.double(weightedC),
                       as.double(cache), 
                       as.double(tol),
+                      as.integer(10), ##qpsize
                       as.integer(shrinking),
-                      .kernelin,
-                      environment(.kernelin), PACKAGE="kernlab")
-        alpha(ret)[p] <- list(resv[-(li+lj+1)])
+                      PACKAGE="kernlab")
+        alpha(ret)[p] <- list(resv)
         ## nonzero alpha*y
         coeff(ret)[p] <- list(alpha(ret)[[p]][alpha(ret)[[p]]>0]*yd[alpha(ret)[[p]]>0])
         ## store SV indexes from current problem for later use in predict
-        alphaindex(ret)[p] <- list(c(indexes[[i]],indexes[[j]])[which(resv[-(li+lj+1)]>0)])
+        alphaindex(ret)[p] <- list(c(indexes[[i]],indexes[[j]])[which(resv>0)])
         ## save the indexes from all the SV in a vector (use unique?)
         svindex <- c(svindex,alphaindex(ret)[[p]])
         ## store betas in a vector 
-        b(ret) <- c(b(ret), resv[li+lj+1])
+        b(ret) <- - sapply(coeff(ret),sum) 
         ## used to reconstruct indexes for the patterns matrix x from "indexes" (really usefull ?)
         problem[p] <- list(c(i,j))
         ##store C  in return object
@@ -326,93 +519,7 @@ function (x,
     }
   } 
 
-if(type(ret) == "nu-classification"){
-  indexes <- lapply(sort(unique(y)), function(kk) which(y == kk))
-    for (i in 1:(nclass(ret)-1)) {
-      jj <- i+1
-      for(j in jj:nclass(ret)) {
-        p <- p+1
-       ##prepare data
-        li <- length(indexes[[i]])
-        lj <- length(indexes[[j]])
-        xd <- matrix(0,(li+lj),dim(x)[2])
-        xdi <- 1:(li+lj) <= li
-        xd[xdi,rep(TRUE,dim(x)[2])] <- x[indexes[[i]],]
-        xd[xdi == FALSE,rep(TRUE,dim(x)[2])] <- x[indexes[[j]],]
-        if(y[indexes[[i]][1]] < y[indexes[[j]]][1])
-          {
-            yd <- c(rep(1,li),rep(-1,lj))
-            if(!is.null(class.weights)){
-            weight <- class.weights[weightlabels[c(i,j)]]
-            wl <- c(1,0)
-            nweights <- 2
-          }
-          }
-        else
-          {
-            yd <- c(rep(-1,li),rep(1,lj))
-            if(!is.null(class.weights)){
-            weight <- class.weights[weightlabels[c(j,i)]]
-            wl <- c(0,1)
-            nweigths <- 2
-          }
-          }
-       
-        boolabel <- yd >= 0
-        prior1 <- sum(boolabel)
-        md <- length(yd)
-        prior0 <- md - prior1
-        prior(ret)[[p]] <- list(prior1 = prior1, prior0 = prior0)
-        
-        resv <- .Call("smo_optim",
-                      as.double(t(xd)),
-                      as.integer(nrow(xd)),
-                      as.integer(ncol(xd)),
-                      as.double(yd),
-                      as.integer(if (sparse) xd@ia else 0),
-                      as.integer(if (sparse) xd@ja else 0),
-                      as.integer(sparse),
-                      
-                      as.double(matrix(rep(-1,m))), #linear term
-                      as.integer(ktype),
-                      as.integer(1),
-                      as.double(C),
-                      as.double(nu),
-                      as.double(epsilon),
-                      as.double(sigma),
-                      as.double(degree),
-                      as.double(offset),
-                      as.integer(wl), #weightlabl.
-                      as.double(weight),
-                      as.integer(nweights),
-                      as.double(cache),
-                      as.double(tol), 
-                      as.integer(shrinking),
-                      .kernelin,
-                      environment(.kernelin), PACKAGE="kernlab")
-        
-        alpha(ret)[p] <- list(resv[-(li+lj+1)])
-        ## alpha*y whithout including zeros (smaller kernel matrixes)
-        coeff(ret)[p] <- list(alpha(ret)[[p]][alpha(ret)[[p]]!= 0])
-        ##store SV indexes from current problem for later use in predict
-         alphaindex(ret)[p] <- list(c(indexes[[i]],indexes[[j]])[which(resv[-(li+lj+1)]!= 0)])
-        ##alphaindex(ret)[p] <- list(c(which(alpha(ret)[[p]][1:li]!=0)+li*(i-1),which(alpha(ret)[[p]][-(1:li)]!=0)+li+lj*(j-2)))
-        ##save the indexes from all the SV in a vector (use unique!)
-               svindex <- c(svindex,alphaindex(ret)[[p]])
-        ## store betas in a vector 
-               b(ret) <- c(b(ret), resv[li+lj+1])
-        ## used to reconstruct indexes for the patterns matrix x from "indexes"
-               problem[p] <- list(c(i,j))
-               param(ret)$nu <- nu
-        
-##        margin(ret)[p] <- (min(kernelMult(kernel,xd[1:li,],,alpha(ret)[[p]][1:li])) - max(kernelMult(kernel,xd[li:(li+lj),],,alpha(ret)[[p]][li:(li+lj)])))/2
-    
-      }
-    }
-  } 
-  
-
-if(type(ret) =="spoc-classification")
+if(type(ret) =="spoc-svc")
   {
     if(!is.null(class.weights))
      weightedC <- class.weights[weightlabels] * rep(C,nclass(ret))
@@ -421,18 +528,23 @@ if(type(ret) =="spoc-classification")
     yd <- sort(y,method="quick", index.return = TRUE)
     x<-x[yd$ix,]
     count <- 0
+
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
+    
     resv <- .Call("tron_optim",
                   as.double(t(x)),
                   as.integer(nrow(x)),
                   as.integer(ncol(x)),
                   as.double(rep(yd$x-1,2)),
+                  as.double(K),
                   as.integer(if (sparse) x@ia else 0),
                   as.integer(if (sparse) x@ja else 0),
                   as.integer(sparse),
                   as.integer(nclass(ret)),
                   as.integer(count),
                   as.integer(ktype),
-                  as.integer(2), 
+                  as.integer(7), 
                   as.double(C),
                   as.double(epsilon),
                   as.double(sigma),
@@ -448,8 +560,7 @@ if(type(ret) =="spoc-classification")
                   as.double(tol),
                   as.integer(10), #qpsize
                   as.integer(shrinking),
-                  .kernelin,
-                  environment(.kernelin), PACKAGE="kernlab")
+                  PACKAGE="kernlab")
 
     alpha(ret) <- t(matrix(resv,nclass(ret)))
     coeff(ret) <- lapply(1:nclass(ret), function(x) alpha(ret)[,x][alpha(ret)[,x]!=0])
@@ -461,7 +572,7 @@ if(type(ret) =="spoc-classification")
     param(ret)$C <- C
   }
 
-if(type(ret) =="kbb-classification")
+if(type(ret) =="kbb-svc")
   {
     if(!is.null(class.weights))
       weightedC <- weightlabels * rep(C,nclass(ret))
@@ -470,24 +581,28 @@ if(type(ret) =="kbb-classification")
     yd <- sort(y,method="quick", index.return = TRUE)
     x<-x[yd$ix,]
     count <-  sapply(unique(yd$x), function(c) length(yd$x[yd$x==c]))
+
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
     resv <- .Call("tron_optim",
                   as.double(t(x)),
                   as.integer(nrow(x)),
                   as.integer(ncol(x)),
                   as.double(yd$x-1),
+                  as.double(K),
                   as.integer(if (sparse) x@ia else 0),
                   as.integer(if (sparse) x@ja else 0),
                   as.integer(sparse),
                   as.integer(nclass(ret)),
                   as.integer(count),
                   as.integer(ktype),
-                  as.integer(1),
+                  as.integer(8),
                   as.double(C),
                   as.double(epsilon),
                   as.double(sigma),
                   as.double(degree),
                   as.double(offset),
-                  as.double(C),
+                  as.double(1), #Cbegin
                   as.double(2), #Cstep
                   as.integer(0), #weightlabl.
                   as.double(0),
@@ -497,41 +612,26 @@ if(type(ret) =="kbb-classification")
                   as.double(tol),
                   as.integer(10), #qpsize
                   as.integer(shrinking),
-                  .kernelin,
-                  environment(.kernelin), PACKAGE="kernlab")
-    start <-rep(0,nclass(ret))
-    start[1]<-0
-    start2<-rep(0,nclass(ret))
-    alpha(ret)<-matrix(0,nrow(x),nclass(ret)-1)
-    start[2:nclass(ret)]<-cumsum(count)[1:nclass(ret)-1]
-    for (i in 2:nclass(ret))
-      start2[i] <- start2[i-1] + nrow(x) - count[i]
-    p<-1
-    se<-1:nclass(ret)
-    for(i in se){
-      for(j in (start[i]+1):(start[i]+count[i]))
-        {
-          for(k in se[se<i])
-            alpha(ret)[p,k] <- resv[start2[k]+j-count[k]]
-          for(k in se[se>i])
-            alpha(ret)[p,k-1] <- resv[start2[k]+j]
-          p <- p+1
-        }
-    }
+                  PACKAGE="kernlab")
+    alpha(ret) <- matrix(resv,nrow(x))
 
     coeff(ret) <-  lapply(1:(nclass(ret)-1), function(x) alpha(ret)[,x][alpha(ret)[,x]!=0])
     alphaindex(ret) <-  lapply(1:(nclass(ret)-1), function(x) which(alpha(ret)[,x]!=0))
     svindex <- which(resv !=0)  ## have to figure out what to do with this...!
-    b(ret) <- 0
+    b(ret) <- - sapply(coeff(ret),sum) 
   }
 
-  if(type(ret) =="one-classification")
+  if(type(ret) =="one-svc")
   {
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
+       
     resv <- .Call("smo_optim",
                   as.double(t(x)),
                   as.integer(nrow(x)),
                   as.integer(ncol(x)),
                   as.double(matrix(rep(1,m))),
+                  as.double(K),
                   as.integer(if (sparse) x@ia else 0),
                   as.integer(if (sparse) x@ja else 0),
                   as.integer(sparse),
@@ -550,8 +650,7 @@ if(type(ret) =="kbb-classification")
                   as.double(cache),
                   as.double(tol),
                   as.integer(shrinking),
-                  .kernelin,
-                  environment(.kernelin), PACKAGE="kernlab")
+                  PACKAGE="kernlab")
     alpha(ret) <- resv[-(m+1)]
     coeff(ret) <- alpha(ret)[alpha(ret)!=0]
     alphaindex(ret) <- which(alpha(ret)!=0) ## in this case and in regr. the same with svindex
@@ -560,13 +659,17 @@ if(type(ret) =="kbb-classification")
     param(ret)$nu <- nu
   }
 
-  if(type(ret) =="eps-regression")
+  if(type(ret) =="eps-svr")
   {
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
+       
      resv <- .Call("smo_optim",
                   as.double(t(x)),
                   as.integer(nrow(x)),
                   as.integer(ncol(x)),
                   as.double(y),
+                   as.double(K),
                   as.integer(if (sparse) x@ia else 0),
                   as.integer(if (sparse) x@ja else 0),
                   as.integer(sparse),
@@ -585,8 +688,7 @@ if(type(ret) =="kbb-classification")
                   as.double(cache), 
                   as.double(tol), 
                   as.integer(shrinking), 
-                  .kernelin,
-                  environment(.kernelin),PACKAGE="kernlab")
+                   PACKAGE="kernlab")
     alpha(ret) <- resv[-(m+1)]
     coeff(ret) <- alpha(ret)[alpha(ret)!=0]
     alphaindex(ret) <- which(alpha(ret)!=0)
@@ -595,13 +697,17 @@ if(type(ret) =="kbb-classification")
     param(ret)$epsilon <- epsilon
   }
 
- if(type(ret) =="nu-regression")
+ if(type(ret) =="nu-svr")
   {
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
+    
     resv <- .Call("smo_optim",
                   as.double(t(x)),
                   as.integer(nrow(x)),
                   as.integer(ncol(x)),
                   as.double(y),
+                  as.double(K),
                   as.integer(if (sparse) x@ia else 0),
                   as.integer(if (sparse) x@ja else 0),
                   as.integer(sparse),
@@ -620,8 +726,7 @@ if(type(ret) =="kbb-classification")
                   as.double(cache), 
                   as.double(tol), 
                   as.integer(shrinking), 
-                  .kernelin,
-                  environment(.kernelin), PACKAGE="kernlab")
+                  PACKAGE="kernlab")
     alpha(ret) <- resv[-(m+1)]
     coeff(ret) <- alpha(ret)[alpha(ret)!=0]
     alphaindex(ret) <- which(alpha(ret)!=0)
@@ -630,6 +735,49 @@ if(type(ret) =="kbb-classification")
     param(ret)$epsilon <- epsilon
     param(ret)$nu <- nu
   }
+
+   if(type(ret) =="eps-bsvr")
+  {
+       if(ktype==4)
+          K <- kernelMatrix(kernel,x)
+       
+     resv <- .Call("tron_optim",
+                  as.double(t(x)),
+                  as.integer(nrow(x)),
+                  as.integer(ncol(x)),
+                  as.double(y),
+                   as.double(K),
+                  as.integer(if (sparse) x@ia else 0),
+                  as.integer(if (sparse) x@ja else 0),
+                  as.integer(sparse),
+                  as.double(2),
+                   as.double(0),
+                  as.integer(ktype),
+                  as.integer(6),
+                   as.double(C),
+                   as.double(epsilon),
+                   as.double(sigma),
+                  as.double(degree),
+                  as.double(offset),
+                   as.double(1),  #Cbegin
+                   as.double(2), #Cstep
+                  as.integer(0), #weightlabl.
+                  as.double(0),
+                  as.integer(0),
+                   as.double(0),
+                  as.double(cache), 
+                  as.double(tol),
+                   as.integer(10), #qpsize
+                  as.integer(shrinking), 
+                   PACKAGE="kernlab")
+    alpha(ret) <- resv
+    coeff(ret) <- alpha(ret)[alpha(ret)!=0]
+    alphaindex(ret) <- which(alpha(ret)!=0)
+    svindex <- which(alpha(ret) !=0) 
+    b(ret) <- -sum(alpha(ret))
+    param(ret)$epsilon <- epsilon
+  }
+
   
   kcall(ret) <- match.call()
   kernelf(ret) <- kernel
@@ -644,9 +792,9 @@ if(type(ret) =="kbb-classification")
     predict(ret, unscaledx) else NA
 
   if (fit){
-    if(type(ret)=="C-classification"||type(ret)=="nu-classification"||type(ret)=="spoc-classification"||type(ret)=="kbb-classification")
+    if(type(ret)=="C-svc"||type(ret)=="nu-svc"||type(ret)=="spoc-svc"||type(ret)=="kbb-svc"||type(ret)=="C-bsvc")
       error(ret) <- 1 - .classAgreement(table(y,as.integer(fit(ret))))
-    if(type(ret)=="eps-regression"||type(ret)=="nu-regression")
+    if(type(ret)=="eps-svr"||type(ret)=="nu-svr"||type(ret)=="eps-bsvr")
       error(ret) <- drop(crossprod(fit(ret) - y)/m)
   }
 
@@ -661,8 +809,8 @@ if(type(ret) =="kbb-classification")
       for(i in 1:cross)
         {
          
-          cind <- unsplit(vgr[-i],1:(m-length(vgr[[i]])))
-          if(type(ret)=="C-classification"||type(ret)=="nu-classification"||type(ret)=="spoc-classification"||type(ret)=="kbb-classification")
+          cind <-  unsplit(vgr[-i],factor(rep((1:cross)[-i],unlist(lapply(vgr[-i],length)))))
+          if(type(ret)=="C-svc"||type(ret)=="nu-svc"||type(ret)=="spoc-svc"||type(ret)=="kbb-svc"||type(ret)=="C-bsvc")
             {
               if(is.null(class.weights))
                 cret <- ksvm(x[cind,],y[cind],type = type(ret),kernel=kernel,kpar = NULL, C=C, nu=nu, tol=tol, scaled=FALSE, cross = 0, fit = FALSE ,cache = cache)
@@ -671,7 +819,7 @@ if(type(ret) =="kbb-classification")
                cres <- predict(cret, x[vgr[[i]],])
             cerror <- (1 - .classAgreement(table(y[vgr[[i]]],as.integer(cres))))/cross + cerror
             }
-          if(type(ret)=="eps-regression"||type(ret)=="nu-regression")
+          if(type(ret)=="eps-svr"||type(ret)=="nu-svr"||type(ret)=="eps-bsvr")
             {
               cret <- ksvm(x[cind,],y[cind],type=type(ret),kernel=kernel,kpar = NULL,C=C,nu=nu,epsilon=epsilon,tol=tol,scaled=FALSE, cross = 0, fit = FALSE, cache = cache, prob.model = FALSE)
               cres <- predict(cret, x[vgr[[i]],])
@@ -687,7 +835,7 @@ if(type(ret) =="kbb-classification")
     {
 
       pres <- NULL
-      if(type(ret)=="C-classification"||type(ret)=="nu-classification")
+      if(type(ret)=="C-svc"||type(ret)=="nu-svc"||type=="C-bsvc")
         {
           p <- 0
           for (i in 1:(nclass(ret)-1)) {
@@ -723,7 +871,7 @@ if(type(ret) =="kbb-classification")
               suppressWarnings(vgr<-split(sample(1:m,m),1:3))
               for(k in 1:3)
                 {
-                  cind <- unsplit(vgr[-k],1:(m-length(vgr[[k]])))
+                  cind <- unsplit(vgr[-k],factor(rep((1:3)[-k],unlist(lapply(vgr[-k],length)))))
                   cret <- ksvm(xd[cind,], yd[cind], type = type(ret),kernel=kernel,kpar = NULL, C=C, nu=nu, tol=tol, scaled=FALSE, cross = 0, fit = FALSE,cache = cache, prob.model=FALSE)
                   pres <- rbind(pres,predict(cret, xd[vgr[[k]],],type="decision"))
                   
@@ -732,11 +880,12 @@ if(type(ret) =="kbb-classification")
             }
           }
         }
-      if(type(ret) == "eps-regression"||type(ret) == "nu-regression"){
+      if(type(ret) == "eps-svr"||type(ret) == "nu-svr"||type(ret)=="eps-bsvr"){
         suppressWarnings(vgr<-split(sample(1:m,m),1:3))
         for(i in 1:3)
           {
-            cind <- unsplit(vgr[-i],1:(m-length(vgr[[i]])))
+            cind <- unsplit(vgr[-i],factor(rep((1:cross)[-i],unlist(lapply(vgr[-i],length)))))
+
             cret <- ksvm(x[cind,],y[cind],type=type(ret),kernel=kernel,kpar = NULL,C=C,nu=nu,epsilon=epsilon,tol=tol,scaled=FALSE, cross = 0, fit = FALSE, cache = cache, prob.model = FALSE)
             cres <- predict(cret, x[vgr[[i]],])
             pres <- rbind(pres,predict(cret, x[vgr[[i]],],type="decision"))
@@ -806,12 +955,10 @@ function (object, newdata, type = "response", coupler = "minpair")
             center = scaling(object)$x.scale$"scaled:center",
             scale  = scaling(object)$x.scale$"scaled:scale"
             )
-
-
  
-  if(type == "response" || type =="decision"||type=="votes")
+  if(type == "response" || type =="decision" || type=="votes")
     {
-  if(type(object)=="C-classification"||type(object)=="nu-classification")
+  if(type(object)=="C-svc"||type(object)=="nu-svc"||type(object)=="C-bsvc")
     {
       predres <- 1:newnrows
       votematrix <- matrix(0,nclass(object),newnrows)
@@ -841,7 +988,7 @@ function (object, newdata, type = "response", coupler = "minpair")
         predres <- sapply(predres, function(x) which.max(votematrix[,x]))
     }
 
-  if(type(object) == "spoc-classification")
+  if(type(object) == "spoc-svc")
     {
       predres <- 1:newnrows
       votematrix <- matrix(0,nclass(object),newnrows)
@@ -850,22 +997,24 @@ function (object, newdata, type = "response", coupler = "minpair")
       predres <- sapply(predres, function(x) which.max(votematrix[,x]))
     }
 
-  if(type(object) == "kbb-classification")
-    {
+  if(type(object) == "kbb-svc")
+    { 
       predres <- 1:newnrows
       votematrix <- matrix(0,nclass(object),newnrows)
       se <- 1:(nclass(object)-1)
       A <- rowSums(alpha(object))
+      Aind <- unique(unlist(alphaindex(object)))
+      sA <- sum(A)
       Aindex <- which(A!=0)
       for(i in 1:nclass(object))
         {
-          for(k in se[se<i])
-            votematrix[k,] <- votematrix[k,] - (kernelMatrix(kernelf(object),newdata,xmatrix(object)[alphaindex(object)[[k]],]) +1)%*%coeff(object)[[k]]
+          for(k in se[se <= i])
+            votematrix[k,] <- votematrix[k,] - kernelMult(kernelf(object), newdata, xmatrix(object)[alphaindex(object)[[k]],],coeff(object)[[k]]) + b(object)[k]
 
-          votematrix[i,] <- votematrix[i,] + (kernelMatrix(kernelf(object),newdata,xmatrix(object)[Aindex,])+1)%*%A[Aindex]      
+          votematrix[i,] <- votematrix[i,] + kernelMult(kernelf(object), newdata, xmatrix(object)[Aind,],A[Aind]) + sA
 
           for(k in se[!se<i])
-            votematrix[k+1,] <- votematrix[k+1,] - (kernelMatrix(kernelf(object),newdata,xmatrix(object)[alphaindex(object)[[k]],]) +1)%*%coeff(object)[[k]]
+            votematrix[k+1,] <- votematrix[k+1,] - kernelMult(kernelf(object), newdata, xmatrix(object)[alphaindex(object)[[k]],],coeff(object)[[k]]) + b(object)[k]
         }
       
       predres <- sapply(predres, function(x) which.max(votematrix[,x]))
@@ -877,7 +1026,7 @@ function (object, newdata, type = "response", coupler = "minpair")
       if(is.null(prob.model(object)[[1]]))
         stop("ksvm object contains no probability model. Make sure you set the paramater prob.model in ksvm during training.")
       
-      if(type(object)=="C-classification"||type(object)=="nu-classification")
+      if(type(object)=="C-svc"||type(object)=="nu-svc"||type(object)=="C-bsvc")
         {
           binprob <- matrix(0, newnrows, nclass(object)*(nclass(object) - 1)/2)
           for(i in 1:(nclass(object)-1))
@@ -892,10 +1041,10 @@ function (object, newdata, type = "response", coupler = "minpair")
           multiprob <- couple(binprob, coupler = coupler)
         }
       else
-        stop("probability estimates only supported for C-classification and nu-classification")
+        stop("probability estimates only supported for C-svc and nu-svc")
     }
   
-  if(type(object) == "one-classification")
+  if(type(object) == "one-svc")
     {
       ret <- kernelMult(kernelf(object),newdata,as.matrix(xmatrix(object)[alphaindex(object),]),coeff(object)) - b(object)
       ret[ret>0]<-1
@@ -903,7 +1052,7 @@ function (object, newdata, type = "response", coupler = "minpair")
       return(ret == 1)      
     }
   else {
-    if(type(object)=="eps-regression"||type(object)=="nu-regression")
+    if(type(object)=="eps-svr"||type(object)=="nu-svr"||type(object)=="eps-bsvr")
       {
         predres <- kernelMult(kernelf(object),newdata,as.matrix(xmatrix(object)[alphaindex(object),]),coeff(object)) - b(object)
       }
@@ -945,27 +1094,41 @@ setMethod("show","ksvm",
 function(object){
   cat("Support Vector Machine object of class \"ksvm\"","\n")
   cat("\n")
-   cat(paste("SV type:", type(object), "\n"))
+   cat(paste("SV type:", type(object)))
+  
   switch(type(object),
-         "C-classification" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
-         "nu-classification" = cat(paste(" parameter : nu =", param(object)$nu, "\n")),
-         "one-classification" = cat(paste(" parameter : nu =", param(object)$nu, "\n")),
-         "spoc-classification" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
-         "eps-regression" = cat(paste(" parameter : epsilon =",param(object)$epsilon,"\n")),
-         "nu-regression" = cat(paste(" parameter : epsilon =", param(object)$epsilon, "nu =", param(object)$nu,"\n"))
+         "C-svc" = cat(paste("  (classification)", "\n")),
+         "nu-svc" = cat(paste("  (classification)", "\n")),
+         "C-bsvc" = cat(paste("  (classification)", "\n")),
+         "one-svc" = cat(paste("  (novelty detection)", "\n")),
+         "spoc-svc" = cat(paste("  (classification)", "\n")),
+         "kbb-svc" = cat(paste("  (classification)", "\n")),
+         "eps-svr" = cat(paste("  (regression)","\n")),
+         "nu-svr" = cat(paste("  (regression)","\n"))
+         )
+  
+  switch(type(object),
+         "C-svc" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
+         "nu-svc" = cat(paste(" parameter : nu =", param(object)$nu, "\n")),
+         "C-bsvc" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
+         "one-svc" = cat(paste(" parameter : nu =", param(object)$nu, "\n")),
+         "spoc-svc" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
+         "kbb-svc" = cat(paste(" parameter : cost C =",param(object)$C, "\n")),
+         "eps-svr" = cat(paste(" parameter : epsilon =",param(object)$epsilon,"\n")),
+         "nu-svr" = cat(paste(" parameter : epsilon =", param(object)$epsilon, "nu =", param(object)$nu,"\n"))
          )
   cat("\n")
  show(kernelf(object))
   cat(paste("\nNumber of Support Vectors :", nSV(object),"\n"))
 
 
-##    if(type(object)=="C-classification" || type(object) == "nu-classification")
+##    if(type(object)=="C-svc" || type(object) == "nu-svc")
 ##      cat(paste("Margin width :",margin(object),"\n"))
   if(!is.null(fit(object)))
     cat(paste("Training error :", round(error(object),6),"\n"))
   if(cross(object)!= -1)
     cat("Cross validation error :",round(cross(object),6),"\n")
-  if(!is.null(prob.model(object)[[1]])&&(type(object)=="eps-regression" ||type(object)=="nu-regression"))
+  if(!is.null(prob.model(object)[[1]])&&(type(object)=="eps-svr" ||type(object)=="nu-svr"||type(object)=="eps-bsvr"))
     cat("Laplace distr. width :",round(prob.model(object)[[1]],6),"\n")
   ##train error & loss
 })
@@ -974,7 +1137,7 @@ function(object){
 setMethod("plot", signature(x = "ksvm", y ="missing"),
 function(x, data = NULL, grid = 50, slice = list(), ...) {
 
-  if (type(x) =="C-classification" || type(x) == "nu-classification") {
+  if (type(x) =="C-svc" || type(x) == "nu-svc") {
     if(nclass(x) > 2)
       stop("plot function only supports binary classification")
   
